@@ -1,3 +1,6 @@
+#ifndef ILIBDEFS_H
+# define ILIBDEFS_H
+
 /* MAX_THREADS removed by Steve Underwood 1999/12/10 */
 
 #ifdef HAVE_CONFIG_H
@@ -8,6 +11,10 @@
 # include <byteswap.h>
 #endif
 
+#ifdef HAVE_READDIR_R
+# define _POSIX_C_SOURCE 199506L
+#endif
+
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
@@ -48,10 +55,10 @@
 #endif
 
 #ifdef STDC_HEADERS
-# include <string.h>
 # include <stdlib.h>
 # include <stdio.h>
 # include <ctype.h>
+# include <string.h>
 #endif
 
 #ifdef HAVE_DIRENT_H
@@ -102,3 +109,5 @@
 # endif
 #endif
 
+#endif /* LIBDEFS_H */
+
