@@ -17,7 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: mcrypt.c,v 1.12 2001/05/17 18:58:20 nmav Exp $ */
+/* $Id: mcrypt.c,v 1.14 2001/09/10 15:59:41 nmav Exp $ */
 
 /* Changed by Steve Underwood 1999/12/10 to allow an arbitrary number of 
  * streams of encryption. Currently the resulting code is probably not
@@ -54,7 +54,7 @@ static int internal_end_mcrypt(MCRYPT td);
 
 static int internal_init_mcrypt(MCRYPT td, void *key, int lenofkey, void *IV)
 {
-	int *sizes = NULL;
+	const int *sizes = NULL;
 	int num_of_sizes, i, ok = 0;
 	int key_size = mcrypt_enc_get_key_size(td);
 
@@ -93,34 +93,43 @@ static int internal_init_mcrypt(MCRYPT td, void *key, int lenofkey, void *IV)
 		key_size = lenofkey;
 	}
 
-	if (sizes!=NULL) free(sizes);
-
 	td->keyword_given = mxcalloc(1, mcrypt_enc_get_key_size(td));
 	if (td->keyword_given==NULL) return MCRYPT_MEMORY_ALLOCATION_ERROR; 
 	
 	memmove(td->keyword_given, key, lenofkey);
 	i = mcrypt_get_size(td);
 	td->akey = mxcalloc(1, i);
-	if (td->akey==NULL) return MCRYPT_MEMORY_ALLOCATION_ERROR;
-
+	if (td->akey==NULL) {
+		free(td->keyword_given);
+		return MCRYPT_MEMORY_ALLOCATION_ERROR;
+	}
 	i = mcrypt_mode_get_size(td);
 	if (i > 0) {
 		td->abuf = mxcalloc(1, i);
-		if (td->abuf==NULL) return MCRYPT_MEMORY_ALLOCATION_ERROR;
+		if (td->abuf==NULL) {
+			free(td->keyword_given);
+			free(td->akey);
+			return MCRYPT_MEMORY_ALLOCATION_ERROR;
+		}
+	}
+	ok = init_mcrypt(td, td->abuf, key, key_size, IV);
+	if (ok!=0) {
+		free(td->keyword_given);
+		free(td->akey);
+		free(td->abuf);
+		return MCRYPT_UNKNOWN_ERROR; /* algorithm error */
 	}
-	init_mcrypt(td, td->abuf, key, key_size, IV);
 
 	ok = mcrypt_set_key(td,
 		       (void *) td->akey,
 		       (void *) td->keyword_given,
 		       key_size, IV, IV!=NULL ? mcrypt_enc_get_iv_size(td) : 0);
 
-	if (ok!=0) return MCRYPT_UNKNOWN_ERROR; /* algorithm error */
-
-	if (td->akey == NULL) {
+	if (ok!=0) {
 		internal_end_mcrypt(td);
-		return MCRYPT_UNKNOWN_ERROR;
+		return MCRYPT_UNKNOWN_ERROR; /* algorithm error */
 	}
+
 	return 0;
 }
 
@@ -167,7 +176,10 @@ int mdecrypt_generic(MCRYPT td, void *ciphertext, int len)
 WIN32DLL_DEFINE
 int mcrypt_generic_end(const MCRYPT td)
 {
-	internal_end_mcrypt(td);
+	if (td==NULL) return MCRYPT_UNKNOWN_ERROR;
+
+	if (td->keyword_given!=NULL)
+		internal_end_mcrypt(td);
 	mcrypt_module_close(td);
 	return 0;
 }
@@ -175,6 +187,8 @@ int mcrypt_generic_end(const MCRYPT td)
 WIN32DLL_DEFINE
 int mcrypt_generic_deinit(const MCRYPT td)
 {
+	if (td==NULL || td->keyword_given==NULL) return MCRYPT_UNKNOWN_ERROR;
+	
 	internal_end_mcrypt(td);
 	return 0;
 }
@@ -209,34 +223,31 @@ void mcrypt_perror(int err)
 }
 
 WIN32DLL_DEFINE
-char* mcrypt_strerror(int err)
+const char* mcrypt_strerror(int err)
 {
-char* cerr=malloc(256);
-	if (cerr==NULL) return NULL;
 
 	switch (err) {
 	case MCRYPT_UNKNOWN_ERROR:
-		sprintf(cerr, "Unknown error.\n");
+		return "Unknown error.\n";
 		break;
 	case MCRYPT_ALGORITHM_MODE_INCOMPATIBILITY:
-		sprintf(cerr,
-			"Algorithm incompatible with this mode.\n");
+		return	"Algorithm incompatible with this mode.\n";
 		break;
 	case MCRYPT_KEY_LEN_ERROR:
-		sprintf(cerr, "Key length is not legal.\n");
+		return "Key length is not legal.\n";
 		break;
 	case MCRYPT_MEMORY_ALLOCATION_ERROR:
-		sprintf(cerr, "Memory allocation failed.\n");
+		return "Memory allocation failed.\n";
 		break;
 	case MCRYPT_UNKNOWN_MODE:
-		sprintf(cerr, "Unknown mode.\n");
+		return "Unknown mode.\n";
 		break;
 	case MCRYPT_UNKNOWN_ALGORITHM:
-		sprintf(cerr, "Unknown algorithm.\n");
+		return "Unknown algorithm.\n";
 		break;
 
 	}
-	return cerr;
+	return NULL;
 }
 
 WIN32DLL_DEFINE
