@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1998,1999 Nikos Mavroyanopoulos 
+ * Copyright (C) 1998,1999,2001 Nikos Mavroyanopoulos 
  *
  * This library is free software; you can redistribute it and/or modify it under the terms of the
  * GNU Library General Public License as published by the Free Software
@@ -17,26 +17,23 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: mcrypt_extra.c,v 1.11 2001/05/18 10:54:14 nmav Exp $ */
+/* $Id: mcrypt_extra.c,v 1.17 2001/09/08 19:06:37 nmav Exp $ */
 
-#ifndef LIBDEFS_H
-#define LIBDEFS_H
 #include <libdefs.h>
-#endif
 #include <bzero.h>
 #include <xmemory.h>
 #include <mcrypt_internal.h>
 
-
 int mcrypt_algorithm_module_ok(const char *file, const char *directory);
 int mcrypt_mode_module_ok(const char *file, const char *directory);
-#ifndef USE_LTDL
-void *mcrypt_dlopen_ext(const char *filename);
-#endif
-void *mcrypt_dlopen(const char *a_directory, const char *m_directory,
+
+void *mcrypt_dlopen(mcrypt_dlhandle *handle, const char *a_directory, const char *m_directory,
 		    const char *filename);
 
-#define MAXPATHLEN 256
+#ifdef HAVE_READDIR_R
+# define MAXPATHLEN 256
+#endif
+
 WIN32DLL_DEFINE char *mcrypt_readdir(DIR * dirstream)
 {
 
@@ -52,10 +49,10 @@ WIN32DLL_DEFINE char *mcrypt_readdir(DIR * dirstream)
 
 #ifdef HAVE_READDIR_R
 		readdir_r(dirstream, ret2, &ret);
-		if (ret==NULL) return NULL;
 #else
 		ret = readdir(dirstream);
 #endif
+		if (ret==NULL) return NULL;
 
 		result = calloc(1, strlen(ret->d_name) + 1);
 		if (result == NULL) {
@@ -72,16 +69,38 @@ WIN32DLL_DEFINE char *mcrypt_readdir(DIR * dirstream)
 
 }
 
+extern const mcrypt_preloaded mps[];
+extern char* strdup(char*);
+
 WIN32DLL_DEFINE char **mcrypt_list_algorithms(char *libdir, int *size)
 {
 	DIR *pdir;
 	char directory[512];
 	char *dirname;
 	char **filename = NULL, *ptr;
-	int tmpsize;
+	int tmpsize, i=0;
+
 
 	*size = 0;
 
+	while( mps[i].name!=0 || mps[i].address!=0) {   
+		if (mps[i].name!=NULL && mps[i].address==NULL) {
+			if (mcrypt_algorithm_module_ok( mps[i].name, NULL) > 0) {
+				filename =
+				    realloc(filename,
+					    (*size +
+					     1) * sizeof(char *));
+				if (filename==NULL) {
+					goto freeall;
+				}
+				filename[*size] = strdup( mps[i].name);
+				if (filename[*size]==NULL) goto freeall;
+				(*size)++;
+			}
+		}
+		i++;
+	}	
+
 	if (libdir == NULL) {
 		strcpy(directory, LIBDIR);
 	} else {
@@ -94,7 +113,7 @@ WIN32DLL_DEFINE char **mcrypt_list_algorithms(char *libdir, int *size)
 		fprintf(stderr, "Unable to open directory %s.\n",
 			directory);
 #endif
-		return NULL;
+		return filename;
 	}
 
 	for (;;) {
@@ -104,23 +123,37 @@ WIN32DLL_DEFINE char **mcrypt_list_algorithms(char *libdir, int *size)
 			if (tmpsize > 3) {
 				if (mcrypt_algorithm_module_ok
 				    (dirname, directory) > 0) {
+
+					ptr =
+					    strrchr( dirname, '.');
+					if (ptr != NULL) {
+						*ptr = '\0';
+						tmpsize = strlen(dirname);
+					}
+					if (_mcrypt_search_symlist_lib( dirname)!=NULL) {
+						free(dirname);
+						continue; /* it's already in the list,
+						           * since it's included in the lib.
+						           */
+					}
 					filename =
 					    realloc(filename,
 						    (*size +
 						     1) * sizeof(char *));
 					if (filename==NULL) {
 						free(dirname);
-						return NULL;
+						goto freeall;
 					}
+
+					
 					filename[*size] =
 					    calloc(1, tmpsize + 1);
-					if (filename[*size]==NULL) return NULL;
+					if (filename[*size]==NULL) {
+						free(dirname);
+						goto freeall;
+					}
 					strcpy(filename[*size], dirname);
 					
-					ptr =
-					    strrchr(filename[*size], '.');
-					if (ptr != NULL)
-						*ptr = '\0';
 					(*size)++;
 				}
 			}
@@ -136,6 +169,12 @@ WIN32DLL_DEFINE char **mcrypt_list_algorithms(char *libdir, int *size)
 
 	return filename;
 
+	freeall:
+		for (i=0;i<(*size);i++) {
+			free(filename[i]);
+		}
+		free(filename);
+		return NULL;
 }
 
 WIN32DLL_DEFINE char **mcrypt_list_modes(char *libdir, int *size)
@@ -145,6 +184,27 @@ WIN32DLL_DEFINE char **mcrypt_list_modes(char *libdir, int *size)
 	char *dirname;
 	char **filename = NULL, *ptr;
 	int tmpsize;
+	int i=0;
+	
+	*size = 0;
+	
+	while( mps[i].name!=0 || mps[i].address!=0) {   
+		if (mps[i].name!=NULL && mps[i].address==NULL) {
+			if (mcrypt_mode_module_ok( mps[i].name, NULL) > 0) {
+				filename =
+				    realloc(filename,
+					    (*size +
+					     1) * sizeof(char *));
+				if (filename==NULL) {
+					goto freeall;
+				}
+				filename[*size] = strdup( mps[i].name);
+				if (filename[*size]==NULL) goto freeall;
+				(*size)++;
+			}
+		}
+		i++;
+	}	
 
 	if (libdir == NULL) {
 		strcpy(directory, LIBDIR);
@@ -158,10 +218,9 @@ WIN32DLL_DEFINE char **mcrypt_list_modes(char *libdir, int *size)
 		fprintf(stderr, "Unable to open directory %s.\n",
 			directory);
 #endif
-		return NULL;
+		return filename;
 	}
 
-	*size = 0;
 	for (;;) {
 
 		dirname = mcrypt_readdir(pdir);
@@ -170,23 +229,35 @@ WIN32DLL_DEFINE char **mcrypt_list_modes(char *libdir, int *size)
 			if (tmpsize > 3) {
 				if (mcrypt_mode_module_ok
 				    (dirname, directory) > 0) {
+
+					ptr =
+					    strrchr( dirname, '.');
+					if (ptr != NULL) {
+						*ptr = '\0';
+						tmpsize = strlen(dirname);
+					}
+					if (_mcrypt_search_symlist_lib( dirname)!=NULL) {
+						free(dirname);
+						continue; /* it's already in the list,
+						           * since it's included in the lib.
+						           */
+					}
 					filename =
 					    realloc(filename,
 						    (*size +
 						     1) * sizeof(char *));
 					if (filename==NULL) {
 						free(dirname);
-						return NULL;
+						goto freeall;
 					}
 					filename[*size] =
 					    calloc(1, tmpsize + 1);
-					if (filename[*size]==NULL) return NULL;
+					if (filename[*size]==NULL) {
+						free(dirname);
+						goto freeall;
+					}
 					
 					strcpy(filename[*size], dirname);
-					ptr =
-					    strrchr(filename[*size], '.');
-					if (ptr != NULL)
-						*ptr = '\0';
 					(*size)++;
 				}
 			}
@@ -199,6 +270,13 @@ WIN32DLL_DEFINE char **mcrypt_list_modes(char *libdir, int *size)
 
 	closedir(pdir);
 	return filename;
+
+	freeall:
+		for (i=0;i<(*size);i++) {
+			free(filename[i]);
+		}
+		free(filename);
+		return NULL;
 }
 
 WIN32DLL_DEFINE void mcrypt_free_p(char **p, int size)
@@ -215,7 +293,8 @@ WIN32DLL_DEFINE
     int mcrypt_algorithm_module_ok(const char *file, const char *directory)
 {
 	word32 ret;
-	lt_dlhandle _handle;
+	mcrypt_dlhandle _handle;
+	void* rr;
 	int (*_version) (void);
 
 	if (file == NULL && directory == NULL) {
@@ -227,27 +306,25 @@ WIN32DLL_DEFINE
 	}
 
 
-/*	LTDL_SET_PRELOADED_SYMBOLS(); */
+	rr = mcrypt_dlopen(&_handle, directory, NULL, file);
 
-	_handle = mcrypt_dlopen(directory, NULL, file);
-
-	if (!_handle) {
+	if (!rr) {
 		lt_dlexit();
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 
-	_version = lt_dlsym(_handle, "_mcrypt_algorithm_version");
+	_version = mcrypt_dlsym(_handle, "_mcrypt_algorithm_version");
 
 	if (_version == NULL) {
-		lt_dlclose(_handle);
+		mcrypt_dlclose(_handle);
 		lt_dlexit();
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 	ret = _version();
 
-	lt_dlclose(_handle);
+	mcrypt_dlclose(_handle);
 	lt_dlexit();
 
 	return ret;
@@ -258,7 +335,8 @@ WIN32DLL_DEFINE
     int mcrypt_mode_module_ok(const char *file, const char *directory)
 {
 	word32 ret;
-	lt_dlhandle _handle;
+	mcrypt_dlhandle _handle;
+	void* rr;
 	int (*_version) (void);
 
 	if (file == NULL && directory == NULL) {
@@ -268,27 +346,25 @@ WIN32DLL_DEFINE
 	if (lt_dlinit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
-/*	LTDL_SET_PRELOADED_SYMBOLS(); */
 
-	_handle = mcrypt_dlopen(directory, NULL, file);
-
-	if (!_handle) {
+	rr = mcrypt_dlopen(&_handle, directory, NULL, file);
+	if (!rr) {
 		lt_dlexit();
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 
-	_version = lt_dlsym(_handle, "_mcrypt_mode_version");
+	_version = mcrypt_dlsym(_handle, "_mcrypt_mode_version");
 
 	if (_version == NULL) {
-		lt_dlclose(_handle);
+		mcrypt_dlclose(_handle);
 		lt_dlexit();
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 	ret = _version();
 
-	lt_dlclose(_handle);
+	mcrypt_dlclose(_handle);
 	lt_dlexit();
 
 	return ret;
