@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1998,1999,2000 Nikos Mavroyanopoulos
+ * Copyright (C) 1998,1999,2000,2001 Nikos Mavroyanopoulos
  *
  * This library is free software; you can redistribute it and/or modify it 
  * under the terms of the GNU Library General Public License as published 
@@ -26,22 +26,93 @@
 #include <xmemory.h>
 
 #ifndef DEBUG
-# define fputs(x, y) 1
+# define fputs(x, y) 
 #endif
 
+extern const mcrypt_preloaded mps[];
+
+#define MAX_MOD_SIZE 1024
+
+static int mcrypt_strcmp( const char* str1, const char* str2) {
+int i;
+int len;
+	if (strlen(str1)!=strlen(str2)) return -1;
+	len = strlen(str1);
+
+	for (i=0;i<len;i++) {
+		if (str1[i]=='_' && str2[i]=='-') continue;
+		if (str2[i]=='_' && str1[i]=='-') continue;
+		if (str1[i]!=str2[i]) return -1;
+	}
+	
+	return 0;
+}
+
+lt_ptr _mcrypt_search_symlist_lib(const char* name) {
+int i=0;
+
+	while( mps[i].name!=0 || mps[i].address!=0) {
+		if (mps[i].name!=NULL && mps[i].address==NULL) {
+			if (mcrypt_strcmp(name, mps[i].name)==0) {
+				 return (void*) -1;
+			}
+		}
+		i++;
+	}
+	return NULL;
+}
+
+lt_ptr _mcrypt_search_symlist_sym(mcrypt_dlhandle handle, const char* _name) {
+int i=0;
+char name[MAX_MOD_SIZE];
+
+	strcpy(name, handle.name);
+	
+	strcat( name, "_LTX_");
+	strcat( name, _name);
+	
+	while( mps[i].name!=0 || mps[i].address!=0) {
+		if (mps[i].name!=NULL) {
+			if (mcrypt_strcmp(name, mps[i].name)==0) {
+				 return mps[i].address;
+			}
+		}
+		i++;
+	}
+	return NULL;
+}
+
+void mcrypt_dlclose( mcrypt_dlhandle handle) {
+void* mod = handle.handle;
+
+	if (mod!=MCRYPT_INTERNAL_HANDLER && mod!=NULL)
+		lt_dlclose(mod);
+	handle.handle = NULL;
+}
+
+lt_ptr mcrypt_dlsym( mcrypt_dlhandle handle, char* str) {
+void* mod;
+
+	mod = handle.handle;
+	
+	if (mod!=MCRYPT_INTERNAL_HANDLER)
+		return lt_dlsym(mod, str);
+	else
+		return _mcrypt_search_symlist_sym(handle, str);
+
+}
+
 WIN32DLL_DEFINE
 int mcrypt_module_close(MCRYPT td)
 {
-
-	lt_dlclose(td->algorithm_handle);
-	lt_dlclose(td->mode_handle);
+	if (td==NULL) return MCRYPT_UNKNOWN_ERROR;
+	
+	mcrypt_dlclose(td->algorithm_handle);
+	mcrypt_dlclose(td->mode_handle);
 	if (lt_dlexit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	td->algorithm_handle = NULL;
-	td->mode_handle = NULL;
-
 	td->m_encrypt = NULL;
 	td->a_encrypt = NULL;
 	td->a_decrypt = NULL;
@@ -52,36 +123,21 @@ int mcrypt_module_close(MCRYPT td)
 	return 0;
 }
 
-#ifndef USE_LTDL
-WIN32DLL_DEFINE
-void *mcrypt_dlopen_ext ( const char *filename) {
-	void *ret;
-	char full_path[1054];
-
-	if (strlen(filename)>sizeof(full_path)) return NULL;
-
-	strcpy(full_path, filename);
-	ret = dlopen(full_path, RTLD_LAZY);
-
-	if (ret==NULL) {
-#ifndef WIN32
-		strcat(full_path, ".so");
-#else
-		strcat(full_path, ".dll");
-#endif
-		ret = dlopen(full_path, RTLD_LAZY);
-	}
-	return ret;
-
-}
-#endif
 
 WIN32DLL_DEFINE
-void *mcrypt_dlopen ( const char* a_directory, const char *m_directory, const char *filename) {
-	void* ret=NULL;
-
-#ifdef USE_LTDL
+void* mcrypt_dlopen ( mcrypt_dlhandle* handle, const char* a_directory, const char *m_directory, const char *filename) {
+	
 	char paths[1526];
+
+	if (strlen(filename) > sizeof( handle->name)) 
+		return MCRYPT_FAILED;
+	else
+		strcpy( handle->name, filename);
+			
+	if (_mcrypt_search_symlist_lib(filename)!=NULL) {
+		handle->handle = MCRYPT_INTERNAL_HANDLER;
+		return handle->handle;
+	}
 	
 	memset( paths, 0, 1024);
 	if (a_directory != NULL) {
@@ -96,37 +152,9 @@ void *mcrypt_dlopen ( const char* a_directory, const char *m_directory, const ch
 
 	lt_dlsetsearchpath(paths);
 
-	ret = lt_dlopenext(filename);
+	handle->handle = lt_dlopenext(filename);
 
-	return ret;
-
-#else
-	char full_path[1050];
-
-	if (a_directory!=NULL) {
-		strncpy(full_path, a_directory, 512);
-		strcat(full_path, "/");
-		strncat( full_path, filename, 512);
-		ret = mcrypt_dlopen_ext(full_path);
-	}
-	if (ret==NULL) {
-		if (m_directory!=NULL) {
-			strncpy(full_path, m_directory, 512);
-			strcat(full_path, "/");
-			strncat( full_path, filename, 510);
-			ret = mcrypt_dlopen_ext(full_path);
-		}
-	}
-	if (ret==NULL) {
-		if (LIBDIR!=NULL) {
-			strncpy(full_path, LIBDIR, 512);
-			strcat(full_path, "/");
-			strncat( full_path, filename, 510);
-			ret = mcrypt_dlopen_ext(full_path);
-		}
-	}
-	return ret;
-#endif
+	return handle->handle;
 
 }
 
@@ -136,6 +164,7 @@ MCRYPT mcrypt_module_open(char *algorithm,
 {
 	MCRYPT td;
 	const char* err;
+	void *ret;
 	
 	td = calloc(1, sizeof(CRYPT_STREAM));
 	if (td==NULL) return MCRYPT_FAILED;
@@ -144,29 +173,31 @@ MCRYPT mcrypt_module_open(char *algorithm,
 		return MCRYPT_FAILED;
 	}
 
-	td->algorithm_handle = mcrypt_dlopen(a_directory, m_directory, algorithm);
-	if (td->algorithm_handle == NULL) {
+	ret = mcrypt_dlopen( &td->algorithm_handle, a_directory, m_directory, algorithm);
+	if (ret == NULL) {
 		err=lt_dlerror();
 		if (err!=NULL) fputs( err, stderr);
 		fputs("\n", stderr);
+		free(td);
 		return MCRYPT_FAILED;
 	}
 
-	td->mode_handle = mcrypt_dlopen(a_directory, m_directory, mode);
+	ret = mcrypt_dlopen( &td->mode_handle, a_directory, m_directory, mode);
 
-	if (td->mode_handle == NULL) {
+	if (ret == NULL) {
 		err=lt_dlerror();
 		if (err!=NULL) fputs( err, stderr);
-		lt_dlclose(td->algorithm_handle);
+		mcrypt_dlclose(td->algorithm_handle);
+		free(td);
 		return MCRYPT_FAILED;
 	}
 
-	td->a_encrypt = lt_dlsym(td->algorithm_handle, "_mcrypt_encrypt");
-	td->a_decrypt = lt_dlsym(td->algorithm_handle, "_mcrypt_decrypt");
-	td->m_encrypt = lt_dlsym(td->mode_handle, "_mcrypt");
-	td->m_decrypt = lt_dlsym(td->mode_handle, "_mdecrypt");
+	td->a_encrypt = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_encrypt");
+	td->a_decrypt = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_decrypt");
+	td->m_encrypt = mcrypt_dlsym(td->mode_handle, "_mcrypt");
+	td->m_decrypt = mcrypt_dlsym(td->mode_handle, "_mdecrypt");
 	td->a_block_size =
-	    lt_dlsym(td->algorithm_handle, "_mcrypt_get_block_size");
+	    mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_block_size");
 
 	if (mcrypt_enc_is_block_algorithm_mode(td) !=
 	    mcrypt_enc_is_block_algorithm(td)) {
@@ -188,7 +219,7 @@ int mcrypt_get_size(MCRYPT td)
 
 	const char *error;
 
-	_mcrypt_get_size = lt_dlsym(td->algorithm_handle, "_mcrypt_get_size");
+	_mcrypt_get_size = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_size");
 	if (_mcrypt_get_size == NULL) {
 		error = lt_dlerror();
 		if (error!=NULL) fputs(error, stderr);
@@ -205,7 +236,7 @@ int mcrypt_mode_get_size(MCRYPT td)
 
 	const char *error;
 
-	_mcrypt_get_size = lt_dlsym(td->mode_handle, "_mcrypt_mode_get_size");
+	_mcrypt_get_size = mcrypt_dlsym(td->mode_handle, "_mcrypt_mode_get_size");
 	if (_mcrypt_get_size == NULL) {
 		error = lt_dlerror();
 		if (error!=NULL) fputs(error, stderr);
@@ -224,7 +255,7 @@ int mcrypt_set_key(MCRYPT td, void *a, void *b, int c, void *d, int e)
 
 	if (mcrypt_enc_is_block_algorithm(td) == 0) {
 		/* stream */
-		__mcrypt_set_key_stream = lt_dlsym(td->algorithm_handle, "_mcrypt_set_key");
+		__mcrypt_set_key_stream = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_set_key");
 		if (__mcrypt_set_key_stream == NULL) {
 			error = lt_dlerror();
 			if (error!=NULL) fputs(error, stderr);
@@ -233,7 +264,7 @@ int mcrypt_set_key(MCRYPT td, void *a, void *b, int c, void *d, int e)
 		}
 		return __mcrypt_set_key_stream(a, b, c, d, e);
 	} else {
-		__mcrypt_set_key_block = lt_dlsym(td->algorithm_handle, "_mcrypt_set_key");
+		__mcrypt_set_key_block = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_set_key");
 		if (__mcrypt_set_key_block == NULL) {
 		 	error = lt_dlerror();
 			if (error!=NULL) fputs(error, stderr);
@@ -250,7 +281,7 @@ int mcrypt_enc_set_state(MCRYPT td, void *iv, int size)
 	int (*__mcrypt_set_state) (void *, void *, int);
 	const char *error;
 
-	__mcrypt_set_state = lt_dlsym(td->mode_handle, "_mcrypt_set_state");
+	__mcrypt_set_state = mcrypt_dlsym(td->mode_handle, "_mcrypt_set_state");
 	if (__mcrypt_set_state==NULL) {
 		error = lt_dlerror();
 		if (error!=NULL) fputs(error, stderr);
@@ -274,7 +305,7 @@ int mcrypt_get_algo_iv_size(MCRYPT td)
 {
 	int (*_mcrypt_get_algo_iv_size) (void);
 
-	_mcrypt_get_algo_iv_size = lt_dlsym(td->algorithm_handle, "_mcrypt_get_algo_iv_size");
+	_mcrypt_get_algo_iv_size = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_algo_iv_size");
 	return _mcrypt_get_algo_iv_size();
 }
 
@@ -293,7 +324,7 @@ int mcrypt_enc_get_key_size(MCRYPT td)
 {
 	int (*_mcrypt_get_key_size) (void);
 
-	_mcrypt_get_key_size = lt_dlsym(td->algorithm_handle, "_mcrypt_get_key_size");
+	_mcrypt_get_key_size = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_key_size");
 	return _mcrypt_get_key_size();
 }
 
@@ -301,13 +332,19 @@ WIN32DLL_DEFINE
 int *mcrypt_enc_get_supported_key_sizes(MCRYPT td, int *len)
 {
 	int *(*_mcrypt_get_key_sizes) (int *);
-	int *size;
+	int *size, *ret;
 
 	_mcrypt_get_key_sizes =
-	    lt_dlsym(td->algorithm_handle, "_mcrypt_get_supported_key_sizes");
+	    mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_supported_key_sizes");
 	size = _mcrypt_get_key_sizes(len);
-
-	return size;
+	
+	ret = NULL;
+	if (size!=NULL) {
+		ret = malloc( sizeof(int)*(*len));
+		if (ret==NULL) return NULL;
+		memcpy( ret, size, sizeof(int)*(*len));
+	}
+	return ret;
 }
 
 WIN32DLL_DEFINE
@@ -315,18 +352,18 @@ int mcrypt_enc_is_block_algorithm(MCRYPT td)
 {
 	int (*_is_block_algorithm) (void);
 
-	_is_block_algorithm = lt_dlsym(td->algorithm_handle, "_is_block_algorithm");
+	_is_block_algorithm = mcrypt_dlsym(td->algorithm_handle, "_is_block_algorithm");
 	return _is_block_algorithm();
 }
 
 WIN32DLL_DEFINE
 char *mcrypt_enc_get_algorithms_name(MCRYPT td)
 {
-	char *(*_mcrypt_get_algorithms_name) (void);
+	const char *(*_mcrypt_get_algorithms_name) (void);
 
 	_mcrypt_get_algorithms_name =
-	    lt_dlsym(td->algorithm_handle, "_mcrypt_get_algorithms_name");
-	return _mcrypt_get_algorithms_name();
+	    mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_algorithms_name");
+	return strdup(_mcrypt_get_algorithms_name());
 }
 
 WIN32DLL_DEFINE
@@ -334,7 +371,7 @@ int init_mcrypt(MCRYPT td, void *buf, void *a, int b, void *c)
 {
 	int (*_init_mcrypt) (void *, void *, int, void *, int);
 
-	_init_mcrypt = lt_dlsym(td->mode_handle, "_init_mcrypt");
+	_init_mcrypt = mcrypt_dlsym(td->mode_handle, "_init_mcrypt");
 	return _init_mcrypt(buf, a, b, c, mcrypt_enc_get_block_size(td));
 }
 
@@ -343,7 +380,7 @@ int end_mcrypt(MCRYPT td, void *buf)
 {
 	int (*_end_mcrypt) (void *);
 
-	_end_mcrypt = lt_dlsym(td->mode_handle, "_end_mcrypt");
+	_end_mcrypt = mcrypt_dlsym(td->mode_handle, "_end_mcrypt");
 	return _end_mcrypt(buf);
 }
 
@@ -370,10 +407,10 @@ int mdecrypt(MCRYPT td, void *buf, void *a, int b)
 WIN32DLL_DEFINE
 char *mcrypt_enc_get_modes_name(MCRYPT td)
 {
-	char *(*_mcrypt_get_modes_name) (void);
+	const char *(*_mcrypt_get_modes_name) (void);
 
-	_mcrypt_get_modes_name = lt_dlsym(td->mode_handle, "_mcrypt_get_modes_name");
-	return _mcrypt_get_modes_name();
+	_mcrypt_get_modes_name = mcrypt_dlsym(td->mode_handle, "_mcrypt_get_modes_name");
+	return strdup(_mcrypt_get_modes_name());
 }
 
 WIN32DLL_DEFINE
@@ -381,7 +418,7 @@ int mcrypt_enc_is_block_mode(MCRYPT td)
 {
 	int (*_is_block_mode) (void);
 
-	_is_block_mode = lt_dlsym(td->mode_handle, "_is_block_mode");
+	_is_block_mode = mcrypt_dlsym(td->mode_handle, "_is_block_mode");
 	return _is_block_mode();
 }
 
@@ -390,7 +427,7 @@ int mcrypt_enc_mode_has_iv(MCRYPT td)
 {
 	int (*_has_iv) (void);
 
-	_has_iv = lt_dlsym(td->mode_handle, "_has_iv");
+	_has_iv = mcrypt_dlsym(td->mode_handle, "_has_iv");
 	return _has_iv();
 }
 
@@ -399,7 +436,7 @@ int mcrypt_enc_is_block_algorithm_mode(MCRYPT td)
 {
 	int (*_is_a_block_mode) (void);
 
-	_is_a_block_mode = lt_dlsym(td->mode_handle, "_is_block_algorithm_mode");
+	_is_a_block_mode = mcrypt_dlsym(td->mode_handle, "_is_block_algorithm_mode");
 	return _is_a_block_mode();
 }
 
@@ -408,7 +445,7 @@ int mcrypt_enc_self_test(MCRYPT td)
 {
 	int (*_self_test) (void);
 
-	_self_test = lt_dlsym(td->algorithm_handle, "_mcrypt_self_test");
+	_self_test = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_self_test");
 
 	return _self_test();
 }
@@ -417,7 +454,8 @@ WIN32DLL_DEFINE
 int mcrypt_module_self_test(char *algorithm, char *a_directory)
 {
 	int i;
-	lt_dlhandle _handle;
+	void* rr;
+	mcrypt_dlhandle _handle;
 	int (*_self_test) (void);
 	const char* error;
 
@@ -425,9 +463,9 @@ int mcrypt_module_self_test(char *algorithm, char *a_directory)
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_handle = mcrypt_dlopen(a_directory, NULL, algorithm);
+	rr = mcrypt_dlopen(&_handle, a_directory, NULL, algorithm);
 
-	if (!_handle) {
+	if (!rr) {
 		error=lt_dlerror();
 		if (error!=NULL) fputs( error, stderr);
 		fputs("\n", stderr);
@@ -435,11 +473,11 @@ int mcrypt_module_self_test(char *algorithm, char *a_directory)
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_self_test = lt_dlsym(_handle, "_mcrypt_self_test");
+	_self_test = mcrypt_dlsym(_handle, "_mcrypt_self_test");
 
 	i = _self_test();
 
-	lt_dlclose(_handle);
+	mcrypt_dlclose(_handle);
 	if (lt_dlexit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
@@ -451,27 +489,28 @@ WIN32DLL_DEFINE
 int mcrypt_module_algorithm_version(char *algorithm, char *a_directory)
 {
 	int i;
-	lt_dlhandle _handle;
+	mcrypt_dlhandle _handle;
 	int (*_version) (void);
 	const char* error;
-
+	void* rr;
+	
 	if (lt_dlinit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_handle = mcrypt_dlopen(a_directory, NULL, algorithm);
-	if (!_handle) {
+	rr = mcrypt_dlopen(&_handle, a_directory, NULL, algorithm);
+	if (!rr) {
 		error=lt_dlerror();
 		if (error!=NULL) fputs( error, stderr);
 		fputs("\n", stderr);
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_version = lt_dlsym(_handle, "_mcrypt_algorithm_version");
+	_version = mcrypt_dlsym(_handle, "_mcrypt_algorithm_version");
 
 	i = _version();
 
-	lt_dlclose(_handle);
+	mcrypt_dlclose(_handle);
 	if (lt_dlexit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
@@ -483,27 +522,28 @@ WIN32DLL_DEFINE
 int mcrypt_module_mode_version(char *mode, char *m_directory)
 {
 	int i;
-	lt_dlhandle _handle;
+	mcrypt_dlhandle _handle;
 	int (*_version) (void);
 	const char* error;
+	void * rr;
 
 	if (lt_dlinit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_handle = mcrypt_dlopen(m_directory, NULL, mode);
-	if (!_handle) {
+	rr = mcrypt_dlopen( &_handle, m_directory, NULL, mode);
+	if (!rr) {
 		error=lt_dlerror();
 		if(error!=NULL) fputs(error, stderr);
 		fputs("\n", stderr);
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_version = lt_dlsym(_handle, "_mcrypt_mode_version");
+	_version = mcrypt_dlsym( _handle, "_mcrypt_mode_version");
 
 	i = _version();
 
-	lt_dlclose(_handle);
+	mcrypt_dlclose(_handle);
 	if (lt_dlexit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
@@ -515,28 +555,28 @@ WIN32DLL_DEFINE
 int mcrypt_module_is_block_algorithm(char *algorithm, char *a_directory)
 {
 	int i;
-	lt_dlhandle _handle;
+	mcrypt_dlhandle _handle;
 	int (*_is_block_algorithm) (void);
 	const char* error;
+	void * rr;
 	
 	if (lt_dlinit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_handle = mcrypt_dlopen(a_directory, NULL, algorithm);
-	
-	if (!_handle) {
+	rr = mcrypt_dlopen( &_handle, a_directory, NULL, algorithm);
+	if (!rr) {
 		error=lt_dlerror();
 		if(error!=NULL) fputs( error, stderr);
 		fputs("\n", stderr);
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_is_block_algorithm = lt_dlsym(_handle, "_is_block_algorithm");
+	_is_block_algorithm = mcrypt_dlsym(_handle, "_is_block_algorithm");
 
 	i = _is_block_algorithm();
 
-	lt_dlclose(_handle);
+	mcrypt_dlclose(_handle);
 	if (lt_dlexit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
@@ -548,27 +588,28 @@ WIN32DLL_DEFINE
 int mcrypt_module_is_block_algorithm_mode(char *mode, char *m_directory)
 {
 	int i;
-	lt_dlhandle _handle;
+	mcrypt_dlhandle _handle;
 	int (*_is_a_block_mode) (void);
 	const char* error;
+	void * rr;
 
 	if (lt_dlinit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_handle = mcrypt_dlopen(m_directory, NULL, mode);
-	if (!_handle) {
+	rr = mcrypt_dlopen( &_handle, m_directory, NULL, mode);
+	if (!rr) {
 		error=lt_dlerror();
 		if(error!=NULL) fputs( error, stderr);
 		fputs("\n", stderr);
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_is_a_block_mode = lt_dlsym(_handle, "_is_block_algorithm_mode");
+	_is_a_block_mode = mcrypt_dlsym(_handle, "_is_block_algorithm_mode");
 
 	i = _is_a_block_mode();
 
-	lt_dlclose(_handle);
+	mcrypt_dlclose(_handle);
 	if (lt_dlexit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
@@ -579,8 +620,9 @@ int mcrypt_module_is_block_algorithm_mode(char *mode, char *m_directory)
 WIN32DLL_DEFINE
 int mcrypt_module_is_block_mode(char *mode, char *m_directory)
 {
+	void * rr;
 	int i;
-	lt_dlhandle _handle;
+	mcrypt_dlhandle _handle;
 	int (*_is_block_mode) (void);
 	const char* error;
 
@@ -588,19 +630,19 @@ int mcrypt_module_is_block_mode(char *mode, char *m_directory)
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_handle = mcrypt_dlopen(m_directory, NULL, mode);
-	if (!_handle) {
+	rr = mcrypt_dlopen( &_handle, m_directory, NULL, mode);
+	if (!rr) {
 		error=lt_dlerror();
 		if(error!=NULL) fputs( error, stderr);
 		fputs("\n", stderr);
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_is_block_mode = lt_dlsym(_handle, "_is_block_mode");
+	_is_block_mode = mcrypt_dlsym(_handle, "_is_block_mode");
 
 	i = _is_block_mode();
 
-	lt_dlclose(_handle);
+	mcrypt_dlclose(_handle);
 	if (lt_dlexit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
@@ -612,27 +654,28 @@ WIN32DLL_DEFINE
 int mcrypt_module_get_algo_block_size(char *algorithm, char *a_directory)
 {
 	int i;
-	lt_dlhandle _handle;
+	mcrypt_dlhandle _handle;
 	int (*_get_block_size) (void);
 	const char* error;
+	void * rr;
 
 	if (lt_dlinit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_handle = mcrypt_dlopen(a_directory, NULL, algorithm);
-	if (!_handle) {
+	rr = mcrypt_dlopen( &_handle, a_directory, NULL, algorithm);
+	if (!rr) {
 		error=lt_dlerror();
 		if(error!=NULL) fputs( error, stderr);
 		fputs("\n", stderr);
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_get_block_size = lt_dlsym(_handle, "_mcrypt_get_block_size");
+	_get_block_size = mcrypt_dlsym(_handle, "_mcrypt_get_block_size");
 
 	i = _get_block_size();
 
-	lt_dlclose(_handle);
+	mcrypt_dlclose(_handle);
 	if (lt_dlexit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
@@ -644,27 +687,28 @@ WIN32DLL_DEFINE
 int mcrypt_module_get_algo_key_size(char *algorithm, char *a_directory)
 {
 	int i;
-	lt_dlhandle _handle;
+	mcrypt_dlhandle _handle;
 	int (*_get_key_size) (void);
 	const char* error;
+	void * rr;
 
 	if (lt_dlinit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_handle = mcrypt_dlopen(a_directory, NULL, algorithm);
-	if (!_handle) {
+	rr = mcrypt_dlopen( &_handle, a_directory, NULL, algorithm);
+	if (!rr) {
 		error=lt_dlerror();
 		if(error!=NULL) fputs( error, stderr);
 		fputs("\n", stderr);
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
-	_get_key_size = lt_dlsym(_handle, "_mcrypt_get_key_size");
+	_get_key_size = mcrypt_dlsym(_handle, "_mcrypt_get_key_size");
 
 	i = _get_key_size();
 
-	lt_dlclose(_handle);
+	mcrypt_dlclose(_handle);
 	if (lt_dlexit() != 0) {
 		return MCRYPT_UNKNOWN_ERROR;
 	}
@@ -677,18 +721,20 @@ int *mcrypt_module_get_algo_supported_key_sizes(char *algorithm,
 						char *a_directory,
 						int *len)
 {
-	lt_dlhandle _handle;
+	mcrypt_dlhandle _handle;
 	int *(*_mcrypt_get_key_sizes) (int *);
 	int *size;
+	int * ret_size;
 	const char* error;
-
+	void * rr;
+	
 	if (lt_dlinit() != 0) {
 		*len = 0;
 		return NULL;
 	}
 
-	_handle = mcrypt_dlopen(a_directory, NULL, algorithm);
-	if (!_handle) {
+	rr = mcrypt_dlopen( &_handle, a_directory, NULL, algorithm);
+	if (!rr) {
 		error=lt_dlerror();
 		if(error!=NULL) fputs( error, stderr);
 		fputs("\n", stderr);
@@ -697,14 +743,20 @@ int *mcrypt_module_get_algo_supported_key_sizes(char *algorithm,
 	}
 
 	_mcrypt_get_key_sizes =
-	    lt_dlsym(_handle, "_mcrypt_get_supported_key_sizes");
+	    mcrypt_dlsym(_handle, "_mcrypt_get_supported_key_sizes");
 
 	size = _mcrypt_get_key_sizes(len);
-
-	lt_dlclose(_handle);
+	if (len!=0) {
+		ret_size = malloc( (*len)*sizeof(int));
+		if (ret_size!=NULL) {
+			memcpy( ret_size, size, (*len)*sizeof(int));
+		}
+	}
+	
+	mcrypt_dlclose(_handle);
 	if (lt_dlexit() != 0) {
 		return NULL;
 	}
 
-	return size;
+	return ret_size;
 }
