@@ -26,7 +26,7 @@
  * All modifications are placed under the license of libmcrypt.
  */
 
-/* $Id: gost.c,v 1.6 2001/05/16 17:54:25 nmav Exp $ */
+/* $Id: gost.c,v 1.9 2001/09/09 09:52:54 nmav Exp $ */
 
 /*
  * If you read the standard, it belabors the point of copying corresponding
@@ -37,8 +37,9 @@
  * lowest-numbered, in the bit string.
  */
 
-#include "../../lib/libdefs.h"
-#include "../../lib/mcrypt_modules.h"
+#include <libdefs.h>
+
+#include <mcrypt_modules.h>
 
 #ifdef USE_LTDL
 #define _mcrypt_set_key gost_LTX__mcrypt_set_key
@@ -338,19 +339,17 @@ WIN32DLL_DEFINE int _mcrypt_get_key_size()
 {
 	return 32;
 }
-WIN32DLL_DEFINE int *_mcrypt_get_supported_key_sizes(int *len)
+
+static const int key_sizes[] = { 32 };
+WIN32DLL_DEFINE const int *_mcrypt_get_supported_key_sizes(int *len)
 {
-	int *size = malloc(1 * sizeof(int));
-	size[0] = 32;
-	*len = 1;
-	return size;
+	*len = sizeof(key_sizes)/sizeof(int);
+	return key_sizes;
+
 }
-WIN32DLL_DEFINE char *_mcrypt_get_algorithms_name()
+WIN32DLL_DEFINE const char *_mcrypt_get_algorithms_name()
 {
-	char *x;
-	x = malloc(10);
-	strcpy(x, "GOST");
-	return x;
+return "GOST";
 }
 
 #define CIPHER "e498cf78cdf1d4a5"
@@ -413,5 +412,5 @@ WIN32DLL_DEFINE int _mcrypt_self_test()
 
 WIN32DLL_DEFINE word32 _mcrypt_algorithm_version()
 {
-	return 19991129;
+	return 20010801;
 }
