@@ -17,8 +17,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "../../lib/libdefs.h"
-#include "../../lib/mcrypt_modules.h"
+#include <libdefs.h>
+#include <mcrypt_modules.h>
 
 #ifdef USE_LTDL
 #define _init_mcrypt ofb_LTX__init_mcrypt
@@ -46,12 +46,23 @@ typedef struct ofb_buf {
 
 int _init_mcrypt( OFB_BUFFER* buf, void *key, int lenofkey, void *IV, int size)
 {
-/* For ofb */
 
+    buf->sd_register = buf->s_register = buf->enc_sd_register = 
+    buf->enc_s_register = NULL;
+    
+    /* For ofb */
 	buf->sd_register=malloc( size);
+    if (buf->sd_register==NULL) goto freeall;
+    
 	buf->s_register=malloc( size);
+    if (buf->s_register==NULL) goto freeall;
+
 	buf->enc_s_register=malloc( size);
+    if (buf->enc_s_register==NULL) goto freeall;
+
 	buf->enc_sd_register=malloc( size);
+    if (buf->enc_sd_register==NULL) goto freeall;
+    
 	if (IV!=NULL) {
 		memmove(buf->sd_register, IV, size);
 		memmove(buf->s_register, IV, size);
@@ -63,6 +74,12 @@ int _init_mcrypt( OFB_BUFFER* buf, void *key, int lenofkey, void *IV, int size)
 
 	return 0;
 
+	freeall:
+		free(buf->sd_register);
+		free(buf->s_register);
+		free(buf->enc_sd_register);
+		free(buf->enc_s_register);
+		return -1;
 }
 
 int _mcrypt_set_state( OFB_BUFFER* buf, void *IV, int size)
@@ -142,9 +159,9 @@ int _mdecrypt( OFB_BUFFER* buf, void *plaintext, int len, int blocksize, void* a
 int _is_block_mode() { return 0; }
 int _has_iv() { return 1; }
 int _is_block_algorithm_mode() { return 1; }
-char *_mcrypt_get_modes_name() { char*x; x=malloc(10); strcpy(x, "OFB"); return x;}
+char *_mcrypt_get_modes_name() { return "OFB"; }
 int _mcrypt_mode_get_size () {return sizeof(OFB_BUFFER);}
 
 word32 _mcrypt_mode_version() {
-	return 20010201;
+	return 20010801;
 }
